from typing import Dict, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class AlfaSbp(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Dict[str, Any]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса AlfaSbp')
        self.__updateCases()

        servicestub.route('/fsCryptoProxy', method='POST', callback=self.entrypoint)

    def entrypoint(self):
        self.log.info("Получен запрос")
        self.__updateCases()

        reqBody = json.loads(bottle.request.body.read())
        command = str(reqBody['command'])
        self.log.info("Метод %s" % command)

        return self.__getResult('/' + command.lower())

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath].get('timeout', 0))

        responseSuffix = self.cases[methodPath].get('responseSuffix', '')

        body = json.dumps(self.cases[methodPath].get('response' + responseSuffix))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")

        return response

    def __updateCases(self):
        self.cases.clear()
        try:
            dirPath = os.path.dirname(os.path.abspath(__file__))
            for file in os.listdir(dirPath):
                if not file.endswith('.json'):
                    continue
                with open(os.path.join(dirPath, file), 'r') as cases:
                    self.cases.update(json.load(cases))
        except Exception as e:
            self.log.error(str(e))


instance = AlfaSbp()
