import json
import os
from configparser import ConfigParser
from services.basicservice import BasicService
from servicestub import ServiceStub
from singleton import Singleton


class DeferredCheck(BasicService, metaclass=Singleton):
    checkStatus = False

    def __init__(self):
        super().__init__(os.path.dirname(os.path.abspath(__file__)))
    
    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса отложенных чеков')
        self._loadCases('cases.json')
        
        conf = ConfigParser()
        conf.read(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'config.ini'))

        if conf.has_option('DeferredCheck', 'checkStatus'):
            self.checkStatus = conf.getboolean('DeferredCheck', 'checkStatus')

        servicestub.route('/CSrest/rest/dictionaries/pendingchecks', method='POST', callback=self.addCheck)
        servicestub.route('/CSrest/rest/dictionaries/pendingchecks/<checkId>', method='GET', callback=self.getCheck)
        servicestub.route('/CSrest/rest/dictionaries/pendingchecks/<checkId>', method='PUT', callback=self.updateCheck)
        servicestub.route('/CSrest/rest/dictionaries/pendingchecks/bypage', method='GET', callback=self.getChecks)
        
    def addCheck(self):
        self.log.info('Добавление отложенного чека')
        methodPath = '/pendingchecks'
        self._loadCases('cases.json')
        
        return self._createResponse(methodPath)

    def getCheck(self, checkId):
        self.log.info(f'Получение отложенного чека: {checkId}')
        methodPath = '/pendingchecks_get'
        self._loadCases('cases.json')

        return self._createResponse(methodPath)
    
    def updateCheck(self, checkId):
        self.log.info(f'Обновление отложенного чека: {checkId}')
        methodPath = '/pendingchecks_put'
        self._loadCases('cases.json')

        return self._createResponse(methodPath)
    
    def getChecks(self):
        self.log.info('Получение списка отложенных чеков')
        methodPath = '/bypage'
        self._loadCases('cases.json')

        response = self._createResponse(methodPath)

        if self.checkStatus == True:
            initialBody = response.body
            content = []
            num = 0

            for deferredCheck in initialBody["content"]:
                if deferredCheck['state'] == 'NOT_PAID':
                    content.append(deferredCheck)
                    num = num + 1

            newBody = initialBody
            newBody['content'] = content
            newBody['totalElements'] = num
            newBody['numberOfElements'] = num
            response.body = newBody

        return response


instance = DeferredCheck()
