#!/usr/bin/env python3

from configparser import ConfigParser
from logger import Logger
from typing import Iterable, Union
import bottle
import importlib
import os
import sys


class ServiceStub(bottle.Bottle):
    def __init__(self):
        super(ServiceStub, self).__init__()
        self.log = Logger()
        conf = ConfigParser()
        conf.read('%s/conf/config.ini' % os.path.dirname(__file__))
        self.host = conf.get('main', 'host') if conf.has_option('main', 'host') else '127.0.0.1'
        self.port = conf.getint('main', 'port') if conf.has_option('main', 'port') else 55555

        self.services = conf.get('main', 'services') if conf.has_option('main', 'services') else ''
        self.services = list(filter(None, map(lambda x: x.strip(), self.services.split(','))))

    def __del__(self):
        self.log.info("Остановка сервиса")

    def start(self):
        self.log.info('Запуск сервиса на %s:%s' % (self.host, self.port))

        # загрузка сервисов.
        # TODO: Сделано на скорую руку, нужно переписать!!!
        for service in self.services:
            self.log.info("Обнаружен сервис %s" % service)
            try:
                module = importlib.import_module('services.%s.%s' % (service, service))
                module.instance.registerRoutes(self)
            except Exception as e:
                self.log.error('Ошибка загрузки модуля %s: %s' % (service, str(e)))

        if len(self.routes):
            self.log.info('Запуск сервиса на %s:%s' % (self.host, self.port))
            self.run(host=self.host, port=self.port)
        else:
            msg = 'Не обнаружено ни одного сервиса'
            print(msg, file=sys.stderr)
            self.log.warning(msg)

    def route(self, paths: Union[str, Iterable[str]], method='GET', callback=None, name=None, apply=None, skip=None, **config):
        paths = [paths] if isinstance(paths, str) else paths
        for path in paths:
            self.log.info("Регистрация метода '%s' по пути '%s'" % (method, path))
            super(ServiceStub, self).route(path, method, callback, name, apply, skip, **config)


if __name__ == '__main__':
    try:
        ServiceStub().start()
    except Exception as e:
        print(e)
